package demo;
/*
 * contactImpl.java
 *
 * Created on April 18, 2007, 6:26 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

import demo.entity.Contact;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;

/**
 *
 * @author Owner
 */
public class ContactImpl implements Contacts {

    private EntityManagerFactory emf;
    private EntityManager em;
    private final String EMPTYSTRING = "";

    /**
     * Creates a new instance of ContactImpl
     */
    public ContactImpl() {

        //emf = Persistence.createEntityManagerFactory("JPAWebServletPU");
        emf = Persistence.createEntityManagerFactory("JPAContactJNDIPU");

        System.out.println("*** in ContactImpl()" + emf);

    }

    /**
     *  Simple method to create and execute a dynamic query
     *  @param firstName  First name of the contact
     *  @param lastName   Last name of the contact
     *  @param phone      Phone number for the contact
     */
    public List<Contact> executeQuery(String firstName, String lastName, String phone) {
        Query q = null;
        List<Contact> result = null;
        boolean paramSpecified = false;
        StringBuffer buf = new StringBuffer("Select c from Contact c where ");
        String query = null;

        if (!firstName.equals(EMPTYSTRING)) {
            paramSpecified = true;
            buf.append("upper(c.firstName) like :firstName and ");
        }
        if (!lastName.equals(EMPTYSTRING)) {
            paramSpecified = true;
            buf.append("upper(c.lastName) like :lastName and ");
        }
        if (!phone.equals(EMPTYSTRING)) {
            paramSpecified = true;
            buf.append("upper(c.phone) like :phone and ");
        }

        // If there are parameters specified, then remove the last "and" otherwise
        // remove "where" from the end of the query.
        if (paramSpecified) {
            query = buf.substring(0, buf.length() - " and".length());
        } else {
            query = buf.substring(0, buf.length() - " where ".length());
        }
        System.out.println("Query is \"" + query + "\"");

        try {
            em = emf.createEntityManager();
            q = em.createQuery(query);

            if (!firstName.equals(EMPTYSTRING)) {
                q = q.setParameter("firstName", "%" + firstName.toUpperCase() + "%");
            }
            if (!lastName.equals(EMPTYSTRING)) {
                q = q.setParameter("lastName", "%" + lastName.toUpperCase() + "%");
            }
            if (!phone.equals(EMPTYSTRING)) {
                q = q.setParameter("phone", "%" + phone.toUpperCase() + "%");
            }

            result = q.getResultList();
            System.out.println("rows returned=" + result.size());
        } finally {
            em.close();
        }
        return result;
    }

    /**
     * Close down the Entity Manager Factory
     */
    public void close() {
        emf.close();
        System.out.println("***closed down Enity Manager Factory***");
    }

    /**
     * Add a new contact to the database
     * @param  contact  The Contact to add
     */
    public void addContact(Contact contact) {
        try {
            System.out.println("***in addContact(): contact:" + contact);
            System.out.println("***emf:" + emf);
            em = emf.createEntityManager();
            System.out.println("***em:" + em);
            System.out.println("***creating tx");
            em.getTransaction().begin();
            em.persist(contact);
            System.out.println("***after persist");
            em.getTransaction().commit();
            System.out.println("row created");

        } finally {
            em.close();
        }
    }

    /**
     * Remove a contact to the database.  The contact that is
     * passed in, needs to be merged prior to removing.
     * @param  contact  The Contact to remove
     */
    public void removeContact(Contact contact) {
        try {
            if (contact != null) {
                em = emf.createEntityManager();
                em.getTransaction().begin();
                Contact mergedContact = em.merge(contact);
                em.remove(mergedContact);
                em.getTransaction().commit();
                System.out.println("row removed");
            }
        } finally {
            em.close();
        }

    }

    /**
     * Update a contact in the database.  The contact that is
     * passed in, needs to be merged prior to the update.
     * @param  contact  The Contact to update
     */
    public void updateContact(Contact contact) {
        try {
            em = emf.createEntityManager();
            em.getTransaction().begin();
            em.merge(contact);
            em.getTransaction().commit();
            System.out.println("row :" + contact + " updated");

        } finally {
            em.close();
        }

    }

    public List<Contact> getContacts() {
        return executeQuery("", "", "");
    }
}
